/*
 * Datamodeller.java    E. L. 2004-02-21
 *
 * Filen inneholder datamodellklasser som kommuniserer med
 * oppussingsprosjektobjektet vrt. Et objekt av en slik klasse danner
 * ei bru mellom GUI-komponenten og oppussingsprosjektobjektet.
 * Oppussingsprosjektobjektet er argument til konstruktren.
 */

/*
 * Her er modellen som ligger bak JList-boksen til hyre i vinduet p figur 15.11.
 * Interfacet ListModel setter betingelsene for datamodellen bak JList.
 * Klassen AbstractListModel implementerer alle metodene unntatt
 * getSize() og getElementAt(). I tillegg trenger vi  lage en metode
 * som srger for  varsle at det er skjedd endringer i oppussingsprosjektobjektet.
 */

import java.util.Formatter;
import javax.swing.*;
import javax.swing.table.*;
import Oppussingsprosjekt.*;

class EgenListeModell extends AbstractListModel {
  private Oppussingsprosjekt prosjektet;

  public EgenListeModell(Oppussingsprosjekt startProsjekt) {
    prosjektet = startProsjekt;
  }

  public int getSize() {
    return prosjektet.finnAntMaterialer();
  }

  public Object getElementAt(int indeks) {
    return prosjektet.finnMateriale(indeks);
  }

  public void registrerNyttElement(Object kilde, int indeks) {
    fireIntervalAdded(kilde, indeks, indeks); // oppdaterer listen p skjermen
  }
}

/*
 * Her er datamodellen som ligger bak JTable-komponenten p figur 15.11.
 * Betingelsene som JTable setter, er gitt av interfacet TableModel.
 * Klassen AbstractTableModel implementerer alle metodene unntatt
 * getColumnCount(), getRowCount() og getValueAt(). Disse m vi implementere i
 * vr subklasse. I tillegg m vi srge for at JTable-komponenten fr tilgang
 * til riktige kolonnenavn ved  lage getColumnName(). Endelig begrenser vi
 * brukerens muligheter ved  lage vr egen isCellEditable()
 */
class EgenTabellModell extends AbstractTableModel {
  private Oppussingsprosjekt prosjektet;

  public EgenTabellModell(Oppussingsprosjekt startProsjekt) {
    prosjektet = startProsjekt;
  }

  public String getColumnName(int indeks) {
    if (indeks >= 0 && indeks < Konstanter.kolonnenavn.length) {
      return Konstanter.kolonnenavn[indeks];
    } else return "";
  }

public boolean isCellEditable(int rad, int kolonne) {
  return false;
}

  public int getRowCount() {
    if (prosjektet == null) return 0;
    else return prosjektet.finnAntFlater();
  }

  public int getColumnCount() {
    return 7;
  }

  public Object getValueAt(int rad, int kolonne) {
    Flate flaten = prosjektet.finnFlate(rad);
    Materiale materialet = flaten.finnMateriale();
    Formatter utformat = new Formatter();
    switch (kolonne) {
      case 0:
        return flaten.finnNavn();
      case 1:
        return utformat.format(Konstanter.formatStreng, flaten.finnBredde());
      case 2:
        return utformat.format(Konstanter.formatStreng, flaten.finnLengde());
      case 3:
        if (materialet != null) {
          String navn = materialet.getClass().getName(); // klassenavn inkl. pakkenavn
          int pos = navn.lastIndexOf('.');
          if (pos >= 0) navn = navn.substring(pos + 1, navn.length());  // bare klassenavnet
          return navn + ": " + materialet.finnNavn();
        } else return null;
      case 4:
        if (materialet != null) {
          return utformat.format(
                     Konstanter.formatStreng, materialet.finnMaterialbehov(flaten));
        } else return null;
      case 5:
        if (materialet != null) {
          return utformat.format(Konstanter.formatStreng, materialet.finnPrisPrEnhet());
        } else return null;
      case 6:
        if (materialet != null) {
          return utformat.format(Konstanter.formatStreng, materialet.finnTotalpris(flaten));
        } else return null;
      default:
        return null;
    }
  }
}
